@@ -1,10 +1,10 @@
 /*
-  Copyright (C) 2000-2006
+  Copyright (C) 2000-2007
 
   Code contributed by Greg Collecutt, Joseph Hope and Paul Cochrane
 
   This file is part of xmds.
- 
+
   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
@@ -21,11 +21,11 @@
 */
 
 /*
-  $Id: main.cc 1361 2007-06-21 01:31:44Z joehope $
+  $Id: main.cc 1597 2007-11-17 13:41:38Z paultcochrane $
 */
 
 /*!
-  @mainpage Documentation for xmds-1.5-4
+  @mainpage Documentation for xmds
 
   @section intro Introduction
 
@@ -42,18 +42,18 @@
 
   @section install Installation
 
-  Download the source tarball from http://www.xmds.org, unpack, and run the 
+  Download the source tarball from http://www.xmds.org, unpack, and run the
   configure script in the xmds directory.<br>
   (as root, to be installed into /usr/local/bin)<br>
   ./configure<br>
   (as a user, to be installed the bin/ directory in your home directory)<br>
   ./configure --with-user<br>
 
-  For more details you can also read the INSTALL file, and even the 
+  For more details you can also read the INSTALL file, and even the
   hand-written documentation.
 */
 
-/*! 
+/*!
   @file main.cc
   @brief The main routine and supporting routines
 
@@ -63,12 +63,10 @@
 // This is the main entry routine for xmds
 
 #include <config.h>
-#include <xmlbasics.h>
-#include <dom3.h>
+#include <xmds_common.h>
 #include <kissdom.h>
-#include <xmlparser.h>
-#include <xmdsutils.h>
-#include <xmdsclasses.h>
+#include <xml_parser.h>
+#include <xmds_simulation.h>
 #include <string>
 #include <cstring>
 #include <cstdio>
@@ -86,31 +84,32 @@ using namespace std;
 //! Displays xmds usage
 void display_usage() {
   cout << "\n" <<
-    "This is xmds version " << VERSION << " release " << RELEASE << " (" << REVISION << ")\n\n" << 
+    "This is xmds version " << VERSION <<
+    " (" << REVISION << ")\n\n" <<
     "      using C compiler " << XMDS_CC;
   if (strcmp(MPICC, "")) {
     cout << " (and C compiler " << MPICC << " for parallel work)\n";
   }
   cout <<
-         "\n"
-         "Usage: xmds [options] infile\n"
-         "Options:\n"
-         "  infile:         required,  The input file\n"
-         "  -h/--help:      optional,  Display this information\n"
-         "  -v/--verbose:   optional,  Verbose mode\n"
-         "  -n/--nocompile: optional,  "
-                           "Turns off automatic compilation of simulation\n"
-         "  -t/--template:  optional,  "
-                           "Outputs an xmds template either to the terminal,\n"
-         "                               or to an optionally specified file\n"
-         "\n"
-         "For further help, please see http://www.xmds.org\n"
-         ;
+    "\n"
+    "Usage: xmds [options] infile\n"
+    "Options:\n"
+    "  infile:         required,  The input file\n"
+    "  -h/--help:      optional,  Display this information\n"
+    "  -v/--verbose:   optional,  Verbose mode\n"
+    "  -n/--nocompile: optional,  "
+    "Turns off automatic compilation of simulation\n"
+    "  -t/--template:  optional,  "
+    "Outputs an xmds template either to the terminal,\n"
+    "                               or to an optionally specified file\n"
+    "\n"
+    "For further help, please see http://www.xmds.org\n"
+    ;
 }
 
 // ********************************************************************
 
-/*! 
+/*!
   @brief Routine to parse the preferences file
 
   @param fPrefs         The input preferences file stream
@@ -128,32 +127,39 @@ void display_usage() {
   @param verbose        Whether or not to print verbose information
   @param debug          Whether or not to print debugging information
 */
-int parsePrefs(ifstream &fPrefs, 
-               string &cc, string &cflags, string &clibs, string &cincludes, 
-               string &cthreadlibs, 
-               string &mpicc, string &mpicflags, 
-               string &fftwlibs, string &fftw_mpi_libs, 
-               string &fftw3libs, string &fftw3threadlibs,
-               bool verbose, bool debug) {
-  
-  /*! 
-     \todo
-     I've thought of a better way to do this, but first I'll just get this
-     version going so that the feature is in, and then I'll go back and make
-     it a bit more elegant.  The idea is to read in each line of the prefs
-     file individually (into a string), and then process the line.  I'll
-     need to read characters until I find an equals sign (ignoring spaces as
-     I go), biff that into macroVar and then grab everything else and put
-     that into macroVarValue.  I could treat the string read in (ie the line
-     I'm parsing) as a stack and pop characters off it until the equals sign
-     is found, putting chars into macroVar and then put the rest (without
-     equals sign into macroVarValue.  Anyway, to be done...
+int parsePrefs(ifstream &fPrefs,
+               string &cc,
+               string &cflags,
+               string &clibs,
+               string &cincludes,
+               string &cthreadlibs,
+               string &mpicc,
+               string &mpicflags,
+               string &fftwlibs,
+               string &fftw_mpi_libs,
+               string &fftw3libs,
+               string &fftw3threadlibs,
+               bool verbose,
+               bool debug) {
 
-     Also, I should make this into a function so that the code is only
-     written the once!
+  /*!
+    \todo
+    I've thought of a better way to do this, but first I'll just get this
+    version going so that the feature is in, and then I'll go back and make
+    it a bit more elegant.  The idea is to read in each line of the prefs
+    file individually (into a string), and then process the line.  I'll
+    need to read characters until I find an equals sign (ignoring spaces as
+    I go), biff that into macroVar and then grab everything else and put
+    that into macroVarValue.  I could treat the string read in (ie the line
+    I'm parsing) as a stack and pop characters off it until the equals sign
+    is found, putting chars into macroVar and then put the rest (without
+    equals sign into macroVarValue.  Anyway, to be done...
+
+    Also, I should make this into a function so that the code is only
+    written the once!
   */
 
-  /*! 
+  /*!
    * \todo this routine should really be part of xmdsUtility, and therefore
    * an instance of an xmdsUtility object (or something like that), because
    * there are far too many parameters being passed into the function, and
@@ -168,25 +174,25 @@ int parsePrefs(ifstream &fPrefs,
   string macroVar, macroVarValue, prefsString;
   bool commentCharFlag;
   prefsString = "";
-  while ( !fPrefs.eof() ) {
+  while (!fPrefs.eof()) {
     // now we try the next line
     macroVar = "";
     macroVarValue = "";
     commentCharFlag = 0;
 
     // wait until we find the equals sign
-    while ( (currentChar = fPrefs.get()) != '=' && 
-           !fPrefs.eof() && 
-           !commentCharFlag) {
+    while ((currentChar = fPrefs.get()) != '=' &&
+            !fPrefs.eof() &&
+            !commentCharFlag) {
       if (currentChar == ' ') {
         if (debug) { cout << "space character before '=' found\n"; }
       }
       else if (currentChar == '#') {
         if (debug) { cout << "comment character found (before '=' found)\n"; }
         commentCharFlag = 1;
-        while ( (currentChar = fPrefs.get()) != '\n' && 
-                !fPrefs.eof() ) {
-          if (debug) { cout << "looping until end of line\n"; } 
+        while ((currentChar = fPrefs.get()) != '\n' &&
+                !fPrefs.eof()) {
+          if (debug) { cout << "looping until end of line\n"; }
         }
         break;
       }
@@ -196,14 +202,14 @@ int parsePrefs(ifstream &fPrefs,
     }
 
     // now loop until we find the return character
-    while ( !commentCharFlag && 
-           (currentChar = fPrefs.get()) != '\n' && 
-           !fPrefs.eof()) {
+    while (!commentCharFlag &&
+            (currentChar = fPrefs.get()) != '\n' &&
+            !fPrefs.eof()) {
       if (currentChar == '#') {
         if (debug) { cout << "comment character found\n"; }
         commentCharFlag = 1;
-        while ( (currentChar = fPrefs.get()) != '\n' && 
-                !fPrefs.eof() ) {
+        while ((currentChar = fPrefs.get()) != '\n' &&
+                !fPrefs.eof()) {
           if (debug) { cout << "looping until end of line\n"; }
         }
         break;
@@ -211,7 +217,7 @@ int parsePrefs(ifstream &fPrefs,
       macroVarValue = macroVarValue + currentChar;
     }
 
-    /*! 
+    /*!
      * \todo this if-elseif sequence can be replaced by a switch-case
      * sequence if the strings are replaced by enums or something similar
      */
@@ -260,7 +266,7 @@ int parsePrefs(ifstream &fPrefs,
       fftw_mpi_libs = macroVarValue;
       if (verbose) { cout << "fftw_mpi_libs set to " + macroVarValue + "\n"; }
     }
-  }  
+  }
   return 0;
 }
 
@@ -272,12 +278,10 @@ int parsePrefs(ifstream &fPrefs,
 */
 void outputTemplate(const char* outfilename) {
 
-  // this is the template text, at present taken directly from the 
+  // this is the template text, at present taken directly from the
   // tutorial, tutTemplateStart.tex of the latex documentation
   // and relevant characters escaped so that they print out properly
-  //! \todo does the template text need to be updated???
-  //! \todo the templateText variable should be a string vector
-  const char* templateText = 
+  const string templateText =
     "<?xml version=\"1.0\"?>\n"
     "<simulation>\n"
     "  \n"
@@ -317,7 +321,7 @@ void outputTemplate(const char* outfilename) {
     "    <name> main </name>\n"
     "    <dimensions> </dimensions> <!-- transverse dims -->\n"
     "    <lattice> </lattice>       <!-- no. of points for each dim -->\n"
-    "    <domains> (,) </domains>   <!-- domain of each dimension -->\n"
+    "    <domains> (, ) </domains>   <!-- domain of each dimension -->\n"
     "    <samples> </samples>       <!-- sample 1st point of dim? -->\n"
     "    \n"
     "    <vector>\n"
@@ -370,7 +374,7 @@ void outputTemplate(const char* outfilename) {
     "  </output>\n"
     "  \n"
     "</simulation>\n";
-    
+
   // check to see if an output file was given
   if (outfilename == 0) {
     // ok, so no input file specified, we therefore spit it out to stdout
@@ -379,30 +383,33 @@ void outputTemplate(const char* outfilename) {
   else if (outfilename != 0) {
     // ok, we have an input file, open it, biff out the string, close it
     // btw, why am I using the old C syntax for this and not C++???
-    //! \todo update file output syntax to C++ completely
-    FILE *fp;
-    fp = fopen(outfilename, "w");
-    if (fp == NULL) {   
+    ofstream templateFile;
+    templateFile.open(outfilename);
+    if (templateFile.fail()) {
       // make sure can actually open the file
       cout << "Unable to open output file: " << outfilename << "\n";
       cout << "Sending output to stdout\n\n";
-      
+
       cout << templateText;
     }
-    else { 
-      /*! 
+    else {
+      /*!
        * \todo the xmds version/copyright info is repeated too often
        * put this into a routine!
        */
-      cout << "This is xmds, version " << VERSION << "-" << RELEASE << " (" << REVISION <<")\n" <<
-        "Copyright 2000-2006 Greg Collecutt, Joseph Hope "<< 
-        "and the xmds-devel team\n" << 
-        "xmds is available from http://www.xmds.org\n\n" << 
+      cout << "This is xmds, version " << VERSION <<
+        " (" << REVISION <<")\n" <<
+        "Copyright 2000-2007 Greg Collecutt, Joseph Hope "<<
+        "and the xmds-devel team\n" <<
+        "xmds is available from http://www.xmds.org\n\n" <<
         "Writing a template to file with filename: " << outfilename << "\n";
-      fprintf(fp, "%s", templateText);
+
+      // actually write the template to the file
+      templateFile << templateText;
+
       cout << "Done!\n";
     }
-    fclose(fp);
+    templateFile.close();
   }
 }
 
@@ -415,7 +422,7 @@ vector<string> simHeaderText;   //!< The text of the xmds script's header
 vector<string> simBodyText;     //!< The text of the xmds script's body
 vector<string> simFooterText;   //!< The text of the xmds script's footer
 
-/*! 
+/*!
   @brief The main routine.
 
   @param argc The number of arguments to the program
@@ -428,55 +435,55 @@ int main(
   bool verbose = 0;
   bool compileFlag = 1;
   bool templateGenFlag = 0;
-  const char* infilename=0;
+  const char* infilename = 0;
 
   int resp;
   while (1) {
-    static struct option long_options[] = 
-    { 
-      {"help", no_argument, 0, 'h'}, 
-      {"verbose", no_argument, 0, 'v'}, 
-      {"debug", no_argument, 0, 'd'}, 
-      {"nocompile", no_argument, 0, 'n'}, 
-      {"template", optional_argument, 0, 't'}, 
-      {"xmldebug", no_argument, 0, 'x'}, 
-      {0,0,0,0} 
-    };
+    static struct option long_options[] =
+      {
+        {"help", no_argument, 0, 'h'},
+        {"verbose", no_argument, 0, 'v'},
+        {"debug", no_argument, 0, 'd'},
+        {"nocompile", no_argument, 0, 'n'},
+        {"template", optional_argument, 0, 't'},
+        {"xmldebug", no_argument, 0, 'x'},
+        {0, 0, 0, 0}
+      };
     int option_index = 0;
     resp = getopt_xmds_long(argc, argv, "hvdnxt", long_options, &option_index);
     if (resp == -1) {
       break;
     }
-    switch (resp) { 
-      case 'h':
-        display_usage();
-        return 0;
-      case 'v':
-        verbose = 1;
-        break;
-      case 'd':
-        debugFlag = 1;
-        break;
-      case 'n':
-        compileFlag = 0;
-        break;
-      case 't':
-        templateGenFlag = 1;
-        break;
-      case 'x':
-        xmlDebugFlag = 1;
-        break;
-      default:
-        display_usage();
-        return 0;
+    switch (resp) {
+    case 'h':
+      display_usage();
+      return 0;
+    case 'v':
+      verbose = 1;
+      break;
+    case 'd':
+      debugFlag = 1;
+      break;
+    case 'n':
+      compileFlag = 0;
+      break;
+    case 't':
+      templateGenFlag = 1;
+      break;
+    case 'x':
+      xmlDebugFlag = 1;
+      break;
+    default:
+      display_usage();
+      return 0;
     }
   }
 
   // process non-option command line elements
   if (optind_xmds < argc) {
     int fnameCount = 0;
-    while (optind_xmds < argc) { 
-      fnameCount++; 
+    while (optind_xmds < argc) {
+      fnameCount++;
       if (fnameCount > 1) {
         // error, input file name already exists
         printf("Error: multiple input files not allowed\n\n");
@@ -493,7 +500,7 @@ int main(
   // if asked to make a template, then just spit it out, either to file
   // or to stdout and then return nicely
   if (templateGenFlag) {
-    /*! \todo 
+    /*! \todo
      * at present, we'll reuse the input file.  I intend to change this to
      * use the getopt_xmds stuff later, so will have to do that at some stage.
      */
@@ -502,17 +509,17 @@ int main(
   }
 
   // check to see that an input file was given
-  if (infilename==0) {
+  if (infilename == 0) {
     // error, no input file was specified
     cout << "Error: no input file specified!\n\n";
     display_usage();
     return 1;
   }
-  
+
   if (verbose) {
     cout << "xmds: inputfile = '" << infilename << "'\n";
   }
-  
+
   // create the XMLParser
   XMLParser myXMLParser;
 
@@ -529,16 +536,16 @@ int main(
     // I tried doing this in vanilla C++, but couldn't
     // it looks like it'll have to be in C for the most part
     FILE *fin;
-    if ((fin = fopen(infilename,"r")) == NULL) {
-      cout << "Can't open the input xmds script file: " << infilename
-        << "Exiting\n"; 
+    if ((fin = fopen(infilename, "r")) == NULL) {
+      cerr << "Can't open the input xmds script file: " << infilename
+           << "Exiting\n";
       return 1;  // and barf
     }
 
     // now grab the file one line at a time
     unsigned char temp;
     string tempString = "";
-    while ( !feof(fin) ) {
+    while (!feof(fin)) {
       temp = fgetc(fin);
       if (temp != '\n') {
         tempString += temp;
@@ -558,12 +565,12 @@ int main(
 
     // now we need to replace all occurrences of " with \"
     for (unsigned int i=0; i<simulationText.size(); i++) {
-      string simulationLine = simulationText[i];
+        string simulationLine = simulationText[i];
       unsigned int lineLen = simulationLine.size();
       for (unsigned int j=0; j<lineLen; j++) {
         if (simulationLine[j] == '\"') {
           if (debugFlag) {
-            cout << "Replaced a double quote at location " << j << 
+            cout << "Replaced a double quote at location " << j <<
               ", lineLen = " << lineLen << "\n";
             cout << "Line is:\n" << simulationLine.c_str() << "\n";
           }
@@ -585,15 +592,15 @@ int main(
     // the header, (ie the <?xml version="1.0"?> (plus possibly more text)
     // and then the <simulation> tag), the body (the rest of the simulationText
     // up until the footer, which is just the </simulation> tag.
-    // This ripping to bits is necessary so that we can piece together the 
+    // This ripping to bits is necessary so that we can piece together the
     // simulation script with the xsil output at the end of the simulation
     // without relying on system() calls.
-    
+
     // search for the text "<simulation>"
     // barf if we get to the end of the file, and still haven't found it.
     string simulationStartTag = "<simulation>";
     string simulationEndTag = "</simulation>";
-    
+
     bool foundSimStartTag = false, foundSimEndTag = false;
 
     // go through the text and rip out the header, body and footer
@@ -623,8 +630,8 @@ int main(
 
     // if we got to here and foundSimStart tag is still false, then barf appropriately
     if (!foundSimStartTag) {
-      cout << "Failed to find the string \"<simulation>\" within the simulation text\n";
-      cout << "Exiting\n";
+      cerr << "Failed to find the string \"<simulation>\" within the simulation text\n";
+      cerr << "Exiting\n";
       // I'm sure we should do something more intelligent here...
       //! \todo use xmdsException objects to handle exceptions/exits properly
       return 1;
@@ -649,10 +656,10 @@ int main(
       }
       cout << "-----------------------------\n";
     }
-      
+
     if (!foundSimEndTag) {
-      cout << "Failed to find the string \"</simulation>\" within the simulation text\n";
-      cout << "Exiting\n";
+      cerr << "Failed to find the string \"</simulation>\" within the simulation text\n";
+      cerr << "Exiting\n";
       // I'm sure we should do something more intelligent here...
       //! \todo use xmdsException objects to handle exceptions/exits properly
       return 1;
@@ -669,16 +676,16 @@ int main(
     }
 
   }
-  catch (XMLParserException XMLRoutinesErr) {
-    cout << "Could not load Document\n" << 
-      "due to the following XMLParserException:\n" << 
-      XMLRoutinesErr.getError() << 
+  catch(XMLParserException XMLRoutinesErr) {
+    cerr << "Could not load Document\n" <<
+      "due to the following XMLParserException:\n" <<
+      XMLRoutinesErr.getError() <<
       "Exiting.\n";
     return 1;
   }
 
   if (*theDocument->documentElement()->nodeName() != "simulation") {
-    cout << "Error: Expecting root element in '" << infilename << 
+    cerr << "Error: Expecting root element in '" << infilename <<
       "' to be <simulation>\n" <<
       "Exiting.\n";
     return 1;
@@ -695,19 +702,19 @@ int main(
 
   // print out some info about xmds
   cout << "This is xmds, version " <<
-         myxmdsSimulation.parameters()->version.c_str() << "-" <<
-         myxmdsSimulation.parameters()->release.c_str() << " (" << REVISION << ")" << "\n";
-  cout << "Copyright 2000-2006 Greg Collecutt, Joseph Hope " <<
-    "and the xmds-devel team\n" << 
+    myxmdsSimulation.parameters()->version.c_str() <<
+    " (" << REVISION << ")" << "\n";
+  cout << "Copyright 2000-2007 Greg Collecutt, Joseph Hope " <<
+    "and the xmds-devel team\n" <<
     "xmds is available from http://www.xmds.org\n\n";
 
   try {
     myxmdsSimulation.processElement(theDocument->documentElement());
   }
-  catch (xmdsException xmdsExceptionErr) {
-    cout << "Error: simulation element could not be processed\n" << 
+  catch(xmdsException xmdsExceptionErr) {
+    cerr << "Error: simulation element could not be processed\n" <<
       "due to the following xmdsException:\n" <<
-      xmdsExceptionErr.getError() << 
+      xmdsExceptionErr.getError() <<
       "Exiting.\n";
     return 1;
   }
@@ -719,9 +726,9 @@ int main(
   try {
     myxmdsSimulation.makeCode(xmdsBytePoint);
   }
-  catch (xmdsException xmdsExceptionErr) {
-    cout << "Error: simulation failed to write output code\n" << 
-      "due to the following xmdsException:\n" << 
+  catch(xmdsException xmdsExceptionErr) {
+    cerr << "Error: simulation failed to write output code\n" <<
+      "due to the following xmdsException:\n" <<
       xmdsExceptionErr.getError() <<
       "Exiting.\n";
     return 1;
@@ -771,11 +778,12 @@ int main(
     homeStuff = "home.stuff";
     findHomeString = "echo $HOME > " + homeStuff;
     system(findHomeString.c_str());
-    
+
     fIn.open(homeStuff.c_str());
     if (fIn.fail()) {
-      // do something
-      //! \todo fail correctly if we can't open the home.stuff file
+      cerr << "Unable to determine the user's home directory\n"
+           << "Exiting\n";
+      return 1;
     }
     fIn >> homeDir;
     fIn.close();
@@ -785,16 +793,16 @@ int main(
     // ~/.xmds/xmds.prefs
     string prefsFname;
     prefsFname = homeDir + "/.xmds/xmds.prefs";
-    
+
     fPrefs.open(prefsFname.c_str());
     if (!fPrefs.fail()) {
       if (verbose) {
         cout << "Prefs file found: " << prefsFname.c_str() << "\n";
       }
       // ok, now try and parse the sucker...
-      parsePrefs(fPrefs, 
-                 cc, cflags, clibs, cincludes, cthreadlibs, 
-                 mpicc, mpicflags, 
+      parsePrefs(fPrefs,
+                 cc, cflags, clibs, cincludes, cthreadlibs,
+                 mpicc, mpicflags,
                  fftwlibs, fftw_mpi_libs, fftw3libs, fftw3threadlibs,
                  verbose, debugFlag);
 
@@ -810,8 +818,9 @@ int main(
         system("echo $PWD > localDir.test");
         fIn.open("localDir.test");
         if (fIn.fail()) {
-          // do something
-          //! \todo fail correctly if we can't open localDir.test
+          cerr << "Unable to properly determine the current directory\n"
+               << "Exiting\n";
+          return 1;
         }
         fIn >> localDir;
         fIn.close();
@@ -823,31 +832,31 @@ int main(
       fPrefs.open(prefsFname.c_str());
       if (!fPrefs.fail()) {
         if (verbose) {
-          cout << "Prefs file found in local directory: " << 
+          cout << "Prefs file found in local directory: " <<
             localDir.c_str() << "\n";
         }
         // ok, now try and parse the sucker...
-        parsePrefs(fPrefs, 
-                   cc, cflags, clibs, cincludes, cthreadlibs, 
-                   mpicc, mpicflags, 
+        parsePrefs(fPrefs,
+                   cc, cflags, clibs, cincludes, cthreadlibs,
+                   mpicc, mpicflags,
                    fftwlibs, fftw_mpi_libs, fftw3libs, fftw3threadlibs,
                    verbose, debugFlag);
 
         fPrefs.close();
       }
-      
+
       // if we get to here, and things have still failed, print a warning
       // and just use the defaults
       else if (fPrefs.fail() && verbose) {
         cout << "Warning: no preferences file found.  "
-          << "Using default values instead\n";
+             << "Using default values instead\n";
       }
     }
 
   }
 
   if (!myxmdsSimulation.parameters()->usePrefs && verbose) {
-    cout << "Warning: User-defined preferences NOT being used.  " << 
+    cout << "Warning: User-defined preferences NOT being used.  " <<
       "Using defaults instead\n";
   }
 
@@ -874,18 +883,21 @@ int main(
 
   if (compileFlag) {
     if (myxmdsSimulation.parameters()->usempi) { //Joe mark
-      if ((myxmdsSimulation.parameters()->nThreads > 1) || 
+      if ((myxmdsSimulation.parameters()->nThreads > 1) ||
           (myxmdsSimulation.parameters()->mpiMethod == "Scheduling")) {
         cout << "compiling for MPI parallel execution with threads...\n";
-        sprintf(command,"%s -D_REENTRANT -o %s %s.cc %s %s %s %s %s %s", 
-            mpicc.c_str(), 
-            myxmdsSimulation.parameters()->simulationName.c_str(), 
-            myxmdsSimulation.parameters()->simulationName.c_str(), 
-            mpicflags.c_str(), 
-            cincludes.c_str(), clibs.c_str(), 
-            myxmdsSimulation.parameters()->fftwVersion==2 ? cthreadlibs.c_str() : fftw3threadlibs.c_str(), 
-            myxmdsSimulation.parameters()->fftwVersion==2 ? fftw_mpi_libs.c_str() : fftw3libs.c_str(),
-						myxmdsSimulation.parameters()->useIntelMKL ? "-lguide -lvml" : "");
+        sprintf(command, "%s -D_REENTRANT -o %s %s.cc %s %s %s %s %s %s",
+                mpicc.c_str(),
+                myxmdsSimulation.parameters()->simulationName.c_str(),
+                myxmdsSimulation.parameters()->simulationName.c_str(),
+                mpicflags.c_str(),
+                cincludes.c_str(),
+                clibs.c_str(),
+                myxmdsSimulation.parameters()->fftwVersion == 2
+                  ? cthreadlibs.c_str() : fftw3threadlibs.c_str(),
+                myxmdsSimulation.parameters()->fftwVersion == 2
+                  ? fftw_mpi_libs.c_str() : fftw3libs.c_str(),
+                myxmdsSimulation.parameters()->useIntelMKL ? "-lguide -lvml" : "");
         cout << "        " << command << "\n";
         if (system(command)) {
           cout << "compilation failed.\n";
@@ -895,13 +907,16 @@ int main(
       else if (!(myxmdsSimulation.parameters()->stochastic)) {
         cout << "compiling for MPI parallel execution for a "
           "nondeterministic simulation...\n";
-        sprintf(command,"%s -o %s %s.cc %s %s %s %s %s", 
-            mpicc.c_str(), 
-            myxmdsSimulation.parameters()->simulationName.c_str(),
-            myxmdsSimulation.parameters()->simulationName.c_str(),
-            mpicflags.c_str(), clibs.c_str(),
-            cincludes.c_str(), myxmdsSimulation.parameters()->fftwVersion==2 ? fftw_mpi_libs.c_str() : fftw3libs.c_str(),
-						myxmdsSimulation.parameters()->useIntelMKL ? "-lguide -lvml" : "");
+        sprintf(command, "%s -o %s %s.cc %s %s %s %s %s",
+                mpicc.c_str(),
+                myxmdsSimulation.parameters()->simulationName.c_str(),
+                myxmdsSimulation.parameters()->simulationName.c_str(),
+                mpicflags.c_str(),
+                clibs.c_str(),
+                cincludes.c_str(),
+                myxmdsSimulation.parameters()->fftwVersion == 2
+                  ? fftw_mpi_libs.c_str() : fftw3libs.c_str(),
+                myxmdsSimulation.parameters()->useIntelMKL ? "-lguide -lvml" : "");
         cout << "        " << command << "\n";
         if (system(command)) {
           cout << "compilation failed.\n";
@@ -910,14 +925,16 @@ int main(
       }
       else {
         printf("compiling for MPI parallel execution ...\n");
-        sprintf(command,"%s -o %s %s.cc %s %s %s %s %s", 
-            mpicc.c_str(),
-            myxmdsSimulation.parameters()->simulationName.c_str(),
-            myxmdsSimulation.parameters()->simulationName.c_str(),
-            mpicflags.c_str(), clibs.c_str(),
-            cincludes.c_str(),
-            myxmdsSimulation.parameters()->fftwVersion==2 ? fftw_mpi_libs.c_str() : fftw3libs.c_str(),
-						myxmdsSimulation.parameters()->useIntelMKL ? "-lguide -lvml" : "");
+        sprintf(command, "%s -o %s %s.cc %s %s %s %s %s",
+                mpicc.c_str(),
+                myxmdsSimulation.parameters()->simulationName.c_str(),
+                myxmdsSimulation.parameters()->simulationName.c_str(),
+                mpicflags.c_str(),
+                clibs.c_str(),
+                cincludes.c_str(),
+                myxmdsSimulation.parameters()->fftwVersion == 2
+                  ? fftw_mpi_libs.c_str() : fftw3libs.c_str(),
+                myxmdsSimulation.parameters()->useIntelMKL ? "-lguide -lvml" : "");
         cout << "        " << command << "\n";
         if (system(command)) {
           cout << "compilation failed.\n";
@@ -928,15 +945,19 @@ int main(
     else {
       if (myxmdsSimulation.parameters()->nThreads > 1) {
         cout << "compiling for threaded parallel execution...\n";
-        sprintf(command,"%s -D_REENTRANT %s -o %s %s.cc %s %s %s %s %s %s",
-            cc.c_str(),cflags.c_str(),
-            myxmdsSimulation.parameters()->simulationName.c_str(),
-            myxmdsSimulation.parameters()->simulationName.c_str(),
-            cincludes.c_str(), clibs.c_str(),
-            myxmdsSimulation.parameters()->fftwVersion == 2? cthreadlibs.c_str() : fftw3threadlibs.c_str(),
-            myxmdsSimulation.parameters()->fftwVersion==2? fftwlibs.c_str() : fftw3libs.c_str(),
-            myxmdsSimulation.parameters()->useOpenMP ? "-openmp" : "",
-						myxmdsSimulation.parameters()->useIntelMKL ? "-lguide -lvml" : "");
+        sprintf(command, "%s -D_REENTRANT %s -o %s %s.cc %s %s %s %s %s %s",
+                cc.c_str(),
+                cflags.c_str(),
+                myxmdsSimulation.parameters()->simulationName.c_str(),
+                myxmdsSimulation.parameters()->simulationName.c_str(),
+                cincludes.c_str(),
+                clibs.c_str(),
+                myxmdsSimulation.parameters()->fftwVersion == 2
+                  ? cthreadlibs.c_str() : fftw3threadlibs.c_str(),
+                myxmdsSimulation.parameters()->fftwVersion == 2
+                  ? fftwlibs.c_str() : fftw3libs.c_str(),
+                myxmdsSimulation.parameters()->useOpenMP ? "-openmp" : "",
+                myxmdsSimulation.parameters()->useIntelMKL ? "-lguide -lvml" : "");
         cout << "        " << command << "\n";
         if (system(command)) {
           cout << "compilation failed.\n";
@@ -945,13 +966,16 @@ int main(
       }
       else {
         cout << "compiling ...\n";
-        sprintf(command,"%s %s -o %s %s.cc %s %s %s %s",
-            cc.c_str(),cflags.c_str(),
-            myxmdsSimulation.parameters()->simulationName.c_str(),
-            myxmdsSimulation.parameters()->simulationName.c_str(),
-            cincludes.c_str(), clibs.c_str(),
-            myxmdsSimulation.parameters()->fftwVersion==2? fftwlibs.c_str() : fftw3libs.c_str(),
-						myxmdsSimulation.parameters()->useIntelMKL ? "-lguide -lvml" : "");
+        sprintf(command, "%s %s -o %s %s.cc %s %s %s %s",
+                cc.c_str(),
+                cflags.c_str(),
+                myxmdsSimulation.parameters()->simulationName.c_str(),
+                myxmdsSimulation.parameters()->simulationName.c_str(),
+                cincludes.c_str(),
+                clibs.c_str(),
+                myxmdsSimulation.parameters()->fftwVersion == 2
+                  ? fftwlibs.c_str() : fftw3libs.c_str(),
+                myxmdsSimulation.parameters()->useIntelMKL ? "-lguide -lvml" : "");
         cout << "        " << command << "\n";
         if (system(command)) {
           cout << "compilation failed.\n";
@@ -961,7 +985,7 @@ int main(
     }
 
     // ok, we're ready to go; tell the user about it
-    cout << "\n" << myxmdsSimulation.parameters()->simulationName.c_str() << 
+    cout << "\n" << myxmdsSimulation.parameters()->simulationName.c_str() <<
       " ready to execute\n";
   }
   return 0;
